import argparse
import asyncio
import logging
from pathlib import Path

from asyncio_taskpool.pool import TaskPool

from feedzgerald.config import read_config
from feedzgerald.feed import process_feed

logging.basicConfig(
    level=logging.INFO,
    format="[%(asctime)s] %(levelname)s - %(message)s",
    datefmt="%H:%M:%S",
)
logging.getLogger("asyncio_taskpool").setLevel(logging.WARNING)
logging.getLogger("httpx").setLevel(logging.WARNING)


def parse_args():
    parser = argparse.ArgumentParser(
        description="Curate your RSS feeds by filtering existing ones"
    )
    parser.add_argument(
        "-c",
        "--config",
        type=Path,
        help="The path to the feedzgerald configuration file",
        required=True,
    )
    return parser.parse_args()


async def main(args: argparse.Namespace):
    config = read_config(filepath=args.config)
    pool = TaskPool()
    pool.map(process_feed, arg_iter=config.feeds, num_concurrent=config.parallelism)
    await pool.gather_and_close()


def run_main():
    args = parse_args()
    asyncio.run(main(args))


if __name__ == "__main__":
    run_main()



import os
from dataclasses import dataclass
from pathlib import Path

import tomllib


@dataclass
class FeedConfiguration:
    name: str
    filename: str
    url: str
    website: str
    title_filter: str | None
    negative_url_filter: str | None
    negative_title_filter: str | None
    description_filter: str | None
    negative_description_filter: str | None
    output_folder: Path


@dataclass
class Config:
    parallelism: int
    feeds: list[FeedConfiguration]


def read_config(filepath: Path) -> Config:
    config = tomllib.load(open(filepath, "rb"))
    core_config = config.get("core", {})
    if output_folder_str := core_config.get("output_folder"):
        output_folder = Path(output_folder_str)
    else:
        output_folder = Path(os.getcwd())
    feed_configs = [
        FeedConfiguration(
            name=conf["name"],
            filename=f"{feed_name}.rss",
            url=conf["url"],
            website=conf["website"],
            title_filter=conf.get("title_filter"),
            negative_title_filter=conf.get("negative_title_filter"),
            description_filter=conf.get("description_filter"),
            negative_description_filter=conf.get("negative_description_filter"),
            negative_url_filter=conf.get("negative_url_filter"),
            output_folder=output_folder,
        )
        for feed_name, conf in config["feeds"].items()
    ]
    parallelism = min(len(feed_configs), core_config.get("parallelism", 5))
    return Config(parallelism=parallelism, feeds=feed_configs)



import logging

import feedparser
import httpx
from feedgen.feed import FeedGenerator

from feedzgerald.config import FeedConfiguration

log = logging.getLogger(__name__)


class FeedProcessor:
    """Download a remote RSS file and produce a local, curated one, by applying filters"""

    def __init__(self, feed_config: FeedConfiguration):
        self.feed_config = feed_config

    async def fetch(self) -> httpx.Response:
        async with httpx.AsyncClient() as http_client:
            return await http_client.get(self.feed_config.url)

    def parse(self, response: httpx.Response) -> feedparser.FeedParserDict:
        return feedparser.parse(response.text)

    def filter(
        self, feed: feedparser.FeedParserDict
    ) -> list[feedparser.FeedParserDict]:
        filtered_entries = []
        for feed_entry in feed.entries:
            keep = True
            if title_filter := self.feed_config.title_filter:
                if title_filter not in feed_entry.title:
                    keep = False
            if negative_title_filter := self.feed_config.negative_title_filter:
                if negative_title_filter in feed_entry.title:
                    keep = False
            if description_filter := self.feed_config.description_filter:
                if description_filter not in feed_entry.description:
                    keep = False
            if (
                negative_description_filter
                := self.feed_config.negative_description_filter
            ):
                if negative_description_filter in feed_entry.description:
                    keep = False
            if negative_url_filter := self.feed_config.negative_url_filter:
                if negative_url_filter in feed_entry.link:
                    keep = False
            if keep:
                log.info(f"[{self.feed_config.name}] Keeping feed {feed_entry.title}")
                filtered_entries.append(feed_entry)
        return filtered_entries

    def generate(self, entries: list[feedparser.FeedParserDict]):
        feed = FeedGenerator()
        feed.id(self.feed_config.url)
        feed.link(href=self.feed_config.website)
        feed.description(f"Filtered feed from {self.feed_config.url}")
        feed.title(self.feed_config.name)
        for entry in entries:
            feed_entry = feed.add_entry()
            feed_entry.id(entry.get("id"))
            feed_entry.title(entry.get("title"))
            feed_entry.link(href=entry.get("link"))
            feed_entry.description(entry.get("description"))
            feed_entry.pubDate(entry.get("published"))
        feed.rss_file(self.feed_config.output_folder / self.feed_config.filename)

    async def run(self):
        log.info(f"Processing feed {self.feed_config.name}")
        response = await self.fetch()
        feed = self.parse(response)
        entries = self.filter(feed)
        filtered_feed = self.generate(entries)
        return filtered_feed


async def process_feed(feed_config: FeedConfiguration):
    processor = FeedProcessor(feed_config=feed_config)
    await processor.run()
